<?php

namespace Teknisa\FIN\Modules\ContaReceber\Services;

use Doctrine\ORM\EntityManager;
use Teknisa\FIN\Modules\ContaReceber\Factory\EnvioAutomaticoFactory;
use Teknisa\FIN\Modules\ContaReceber\Services\TitulosAReceberService;

/**
 * Class EnvioAutomaticoService
 * @package Teknisa\FIN\Modules\ContaReceber\Services
 */
class EnvioAutomaticoService {
    
    const ENDERECO_COBRANCA = 'C';

    protected $entityManager;
    protected $envioAutomaticoFactory;
    protected $titulosAReceberService;

    /**
     * @param EntityManager          $entityManager
     * @param EnvioAutomaticoFactory $envioAutomaticoFactory
     * @param TitulosAReceberService $titulosAReceberService
     */
    function __construct(EntityManager $entityManager, EnvioAutomaticoFactory $envioAutomaticoFactory, TitulosAReceberService $titulosAReceberService){
        $this->entityManager          = $entityManager;
        $this->envioAutomaticoFactory = $envioAutomaticoFactory;
        $this->titulosAReceberService = $titulosAReceberService;
    }

    /**
     * Envio Automático de Título por E-mail
     */
    public function envioAutomaticoTituloPorEmail($nrorg, $request, $response){
        try {
            // Busca Parâmetros de Títulos a Receber
            $geralrec = $this->envioAutomaticoFactory->buscaGeralRec($nrorg);
            if($geralrec){
                // Verifica utilização de Envio Automatico de Título por E-mail
                if($geralrec->getIdutenvbolauto() == 'S'){
                    // Busca Títulos para Envio Automatico por E-mail
                    $titulos = $this->envioAutomaticoFactory->buscaEnvioTitulo($geralrec->getQtdiaantenvbol(), $geralrec->getCdtpbairenvbol());
                
                    if($titulos){
                        foreach($titulos as $titulo){
                            // Informações do Título
                            echo('Unidade: '.$titulo['CDFILIAL'].' Titulo: '.$titulo['NRLANBOLREC'].' Vencimento: '.$titulo['DTVENBOLREC'].' Cliente: '.$titulo['CDCLIENTE'].' Valor: '.$titulo['VRFINALBREC'].'<br/>');
                            
                            // Valida Título
                            $erro = $this->validaTitulo($titulo);
                            
                            // Busca Remetente
                            $remetente    = $this->buscaRemetente($titulo['CDFILIAL']);
                            
                            // Busca Destinatários
                            $destinatario = $this->buscaDestinatario($titulo['CDCLIENTE']);
                            
                            if($remetente && $destinatario && !$erro){
                                $anexo = $this->titulosAReceberService->geraRelatorioBoleto($titulo, $request, $response);
                            
                                if (!empty($anexo)) {
                                    $assunto  = '[EMPRESA] - Emissão de Boleto Bancário - Vencimento [VENCIMENTO]';
                                    $mensagem = 'Prezado(a) Senhor(a): [CLIENTE]<br/><br/>' .
                                                'A [EMPRESA] vem por meio deste comunicar a emissão de um novo boleto bancário para V.Sa.<br/><br/>' .
                                                'Para imprimir, abra o documento em anexo no e-mail.<br/><br/>' .
                                                'Em caso de dúvidas entre em contato conosco.<br/><br/>' .
                                                'Atenciosamente,<br/>' .
                                                '[EMPRESA]';
                                    $anexo    = [];
                                    
                                    $this->titulosAReceberService->enviaEmailTitulo($remetente, $dsemailclie, $assunto, $mensagem, $anexo, $titulo);
                                    
                                    $this->excluiRelatorioGerados($caminhoBoletos);
                                }
                            }
                        }
                        echo('Título(s) enviado(s) com sucesso.'.'<br/>');
                    }else echo('Não há título(s) haptos para envio.'.'<br/>');
                }
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Envio Automático de Carta de Cobrança por E-mail
     */
    public function envioAutomaticoCartaCobrancaPorEmail($nrorg){
        try {
            // Busca Parâmetros de Títulos a Receber
            $geralrec = $this->envioAutomaticoFactory->buscaGeralRec($nrorg);
            if($geralrec){
                // Verifica utilização de Envio Automático de Carta de Cobrança por E-mail
                if($geralrec->getIdutenvcartauto() == 'S'){
                    // Busca Títulos para Envio Automático de Carta de Cobrança por E-mail
                    $titulos = $this->envioAutomaticoFactory->buscaEnvioCartaCobranca($geralrec->getQtdiaposenvcar(), $geralrec->getCdtpbairenvbol());
                
                    if($titulos){
                        foreach($titulos as $titulo){
                            // Busca Remetente
                            $remetente    = $this->buscaRemetente($titulo['CDFILIAL']);
                            
                            // Busca Destinatários
                            $destinatario = $this->buscaDestinatario($titulo['CDCLIENTE']);
                            
                            if($remetente && $destinatario){
                                $assunto  = '[EMPRESA] - Carta de Cobrança - Vencimento [VENCIMENTO]';
                                $mensagem = '[MUNICIPIO], [DIA] de [MES] de [ANO].<br/><br/>' .
                                            'Prezado(a) Sr(a) [CLIENTE],<br/><br/>' .
                                            'Consta em nossos registros que o pagamento referente ao título com vencimento para o dia [VENCIMENTO], no valor de [VALOR], encontra-se em aberto.<br/><br/>' .
                                            'Desta forma, fazemos uso do presente para cientificá-lo(a) do débito pendente e solicitar sua mais breve regularização.<br/><br/>' .
                                            'Qualquer dúvida, pedimos a gentileza de entrar em contato conosco para juntos resolvemos esta situação.<br/><br/>' .
                                            'Atenciosamente,<br/>' .
                                            '[EMPRESA]';
                                $anexo    = [];
                                
                                $this->titulosAReceberService->enviaEmailTitulo($remetente, $destinatario, $assunto, $mensagem, $anexo, $titulo);
                            }
                        }
                        echo('Carta(s) de cobrança enviada(s) com sucesso.'.'<br/>');
                    }else echo('Não há títulos haptos para envio de carta de cobrança.'.'<br/>');
                }
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * Busca Remetente
     */
    public function buscaRemetente($cdfilial){
        $remetente = null;
        
        // Busca Endereço Unidade
        $endefili = $this->envioAutomaticoFactory->buscaEndefili($cdfilial, self::ENDERECO_COBRANCA);
        
        if($endefili){
            if($endefili->getDsemailfili())
                $remetente = $endefili->getDsemailfili();
            else echo('Operação bloqueada. Unidade sem email cadastrado.'.'<br/>');
        }else echo('Operação bloqueada. Unidade sem endereço cadastrado.'.'<br/>');
        
        return $remetente;
    }
    
    /**
     * Busca Destinatario
     */
    public function buscaDestinatario($cdcliente){
        $destinatario = null;
        
        // Busca Endereço Cliente
        $endeclie = $this->envioAutomaticoFactory->buscaEndeclie($cdcliente, self::ENDERECO_COBRANCA);
        
        if($endeclie){
            if($endeclie->getDsemailclie())
                $destinatario = $endeclie->getDsemailclie();
            else echo('Operação bloqueada. Cliente sem email cadastrado.'.'<br/>');
        }else echo('Operação bloqueada. Cliente sem endereço cadastrado.'.'<br/>');
        
        return $destinatario;
    }
    
    /**
     * Valida Titulo para Envio Automático por E-mail
     */
    public function validaTitulo($titulo){
        $erro = false;
        
        if ($titulo['CDREDUCONT']){
            // Busca Conta Corrente
            $contcorr = $this->envioAutomaticoFactory->buscaContcorr($titulo['CDREDUCONT']);
            
            if(!$contcorr->getCdcedenteban()){
                echo('Operação bloqueada. Conta corrente sem cedente cadastrado.'.'<br/>');
                $erro = true;
            }
        }else{
            echo('Operação bloqueada. Título sem conta corrente cadastrada.'.'<br/>');
            $erro = true;
        }
        
        if (!$titulo['CDCARTCOBBOL']){
            echo('Operação bloqueada. Título sem carteira cadastrada.'.'<br/>');
            $erro = true;
        }
        
        return $erro;
    }
    

}